'Demo for the VS1003 based MP3 player
'Copyright (C) 2012 Rayslogic.com, LLC
'Note:  You must change the pin settings in this file: VS1003_Driver1d.spin
'       before use to match your setup.
'       You may also have to adjust the SD card settings in the CON section in this file. 

'This example plays a midi then a MP3 file located on SD card 

CON  'Propeller system clock settings
    _clkmode = xtal1 + pll16x                           
    _xinfreq = 5_000_000

CON 'SD card pin settings
  SD_Do=0
  SD_Clk=1
  SD_Di=2
  SD_Cs=3


DAT  'Specify files to play here
  filename1
        byte "feist.mp3",0
  filename2
        byte "FurElise.mid",0  

    
OBJ
  mp3:  "VS1003_Driver1d.spin"
  sd: "fsrw"

CON 'Buffers Size                          
  PlayBufferBlocks = mp3#PlayBufferBlocks  
  PlayBufferLongs= PlayBufferBlocks*8
  PlayBufferBytes= PlayBufferLongs*4  

VAR  'Buffers for SD playback
  long MP3_Buffer1[PlayBufferLongs+1]
  long MP3_Buffer2[PlayBufferLongs+1] 



PUB Main|i

  'Start the VS1003 driver
  mp3.start  
  'turn on both LEDs while opening SD and MP3 file
  mp3.SetLEDs(true,true) '(red led, green led)

  'Try forever to mount SD card
  repeat   
     i:=sd.mount_explicit(SD_DO, SD_CLK, SD_DI,SD_CS)
     if i==0
       quit
     sd.unmount
     waitcnt(cnt+clkfreq/4)

  'turn off LEDs   
  mp3.SetLEDs(false,false) '(red led, green led)    

  'play files
  repeat
    PlaySdFile(@filename1)
    PlaySdFile(@filename2) 


PUB PlaySdFile(pFilename)|i  'Play a file from SD card
  'Need to soft reset
  mp3.SoftReset  'need to do a soft reset before playing a file

  'Set volume to midrange
  mp3.SetVolume(254,254) 'Volume range is 0 to 254 (255 for analog powerdown mode)

  'Open the file
  sd.popen(pFilename,"r") 'open file for reading   
  
  'turn on green led when playing   
  mp3.SetLEDs(false,true) '(red led, green led)   
  
  'Play the MP3
  'first, set buffer status to both empty
  MP3_Buffer1[0]:=0
  MP3_Buffer2[0]:=0

  'next, tell driver to start playing from buffers
  mp3.Play(@MP3_Buffer1,@MP3_Buffer2)

  'now, keep filling buffers until end of file
  repeat     'keep buffers filled
    'wait for buffer#1 to empty, clear it and then fill it
    repeat until MP3_Buffer1[0]==0
    LongFill(@MP3_Buffer1+4,0,PlayBufferLongs)
    i:= sd.pRead(@MP3_Buffer1+4,PlayBufferBytes)
    'mark buffer #1 as filled
    MP3_Buffer1[0]:=i
    if i<0  'end of file?
       quit

    'wait for buffer#2 to empty, clear it and then fill it    
    repeat until MP3_Buffer2[0]==0
    LongFill(@MP3_Buffer2+4,0,PlayBufferLongs) 
    i:=sd.pRead(@MP3_Buffer2+4,PlayBufferBytes)
    'mark buffer #2 as filled 
    MP3_Buffer2[0]:=i
    if i<0   'end of file?  
       quit

  'Close the file     
  sd.pclose

  'Clear the on-chip Play Buffer
  mp3.FlushBuffer
  
  'turn all leds off when done   
  mp3.SetLEDs(false,false) '(red led, green led)



DAT 'Wave header and buffer
header byte 
        byte $52, $49, $46, $46, $1c, $10, $00, $00
        byte $57, $41, $56, $45, $66, $6d, $74, $20 '/*|RIFF....WAVEfmt |*/
        byte $14, $00, $00, $00, $11, $00, $01, $00 '
        byte $40, $1f, $00, $00, $75, $12, $00, $00 '/*|........@......|*/
        byte $00, $01, $04, $00, $02, $00, $f9, $01 '
        byte $66, $61, $63, $74, $04, $00, $00, $00 '/*|.......fact....|*/
        byte $5c, $1f, $00, $00, $64, $61, $74, $61
        byte $e8, $0f, $00, $00

WaveBuffer byte  0[512]'; /* data buffer for saving to disk */

PUB RecordWave   |i,w,idx  ,line_in,nsamples,nloops512,chunksize,filesize

  nloops512:=200
  nsamples:=nloops512*512
  filesize:=nsamples+60
  chunksize:=filesize-8
    
  header[48]:=nsamples&$FF
  header[49]:=(nsamples>>8)&$FF
  header[50]:=(nsamples>>16)&$FF
  header[51]:=(nsamples>>24)&$FF

  header[56]:=nsamples&$FF
  header[57]:=(nsamples>>8)&$FF
  header[58]:=(nsamples>>16)&$FF
  header[59]:=(nsamples>>24)&$FF
    
  header[4]:=chunksize&$FF
  header[5]:=(chunksize>>8)&$FF
  header[6]:=(chunksize>>16)&$FF
  header[7]:=(chunksize>>24)&$FF
  

  line_in:=false

  repeat 5
     sd.mount_explicit(SD_DO, SD_CLK, SD_DI,SD_CS)  
     i:=sd.popen(@filename1,"w") 'open file for writing
     if i==0
       quit
     sd.unmount
     waitcnt(cnt+clkfreq/4)
  if i<>0
    repeat


  
  mp3.WriteRegister(mp3#SCI_VOL,$FEFE)'1414)   '; /* Recording monitor volume */   
  mp3.WriteRegister(mp3#SCI_BASS, 0)'; /* Bass/treble disabled */
  mp3.WriteRegister(mp3#SCI_CLOCKF, $4430)'; /* 2.0x 12.288MHz */
  waitcnt(cnt+100*(clkfreq/1000))'Wait(100);
  mp3.WriteRegister(mp3#SCI_AICTRL0, 12)'; /* Div -> 12=8kHz 8=12kHz 6=16kHz */
  waitcnt(cnt+100*(clkfreq/1000))'Wait(100);
  mp3.WriteRegister(mp3#SCI_AICTRL1, 0)'; /* Auto gain */
  waitcnt(cnt+100*(clkfreq/1000))'Wait(100);
  if (line_in==true) '{
    mp3.WriteRegister(mp3#SCI_MODE, $5804)'; /* Normal SW reset + other bits */
  else '{
    mp3.WriteRegister(mp3#SCI_MODE, $1804)'; /* Normal SW reset + other bits */

  repeat idx from 0 to 59'(idx=0; idx < sizeof(header); idx++) { /* Save header first */
    WaveBuffer[idx] := header[idx]';


  mp3.WriteRegister($07,$C017) 
  mp3.WriteRegister($06,%1000) 
  mp3.WriteRegister($07,$C019)
  mp3.WriteRegister($06,%1000) 

'/* Fix rate if needed */
'/*db[24] = rate;*/
'/*db[25] = rate>>8;*/
'/* Record loop */
  repeat nloops512'50'while (recording_on)' {
    repeat
      w := mp3.ReadRegister(mp3#SCI_HDAT1)';
    while (w < 256) or (w => 896)'; /* wait until 512 bytes available */
    repeat while (idx < 512) '{
      w := mp3.ReadRegister(mp3#SCI_HDAT0)';
      WaveBuffer[idx++] := w>>8
      WaveBuffer[idx++] := w&$FF
    idx := 0
    sd.pwrite(@WaveBuffer,512)'write_block(datasector++, db); /* Write output block to disk */



'... /* Fix WAV header information */
'... /* Then update FAT information */
'ResetMP3(); /* Normal reset, restore default settings */
'SetMp3Vol(vol);
  sd.pclose 

  mp3.SoftReset

  

  mp3.WriteRegister($07,$C017) 
  mp3.WriteRegister($06,%0000) 
  mp3.WriteRegister($07,$C019)
  mp3.WriteRegister($06,%0000) 



    